#!/bin/bash

# MYP Frontend Installations-Skript
# Dieses Skript installiert das Frontend mit Docker und Host-Netzwerkanbindung

# Farbcodes für Ausgabe
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[0;33m'
BLUE='\033[0;34m'
NC='\033[0m' # No Color

# Funktion zur Ausgabe mit Zeitstempel
log() {
  echo -e "${BLUE}[$(date +'%Y-%m-%d %H:%M:%S')]${NC} $1"
}

error_log() {
  echo -e "${RED}[$(date +'%Y-%m-%d %H:%M:%S')] FEHLER:${NC} $1" >&2
}

# Pfade definieren
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
FRONTEND_DIR="$SCRIPT_DIR/packages/reservation-platform"

# Prüfen ob Verzeichnis existiert
if [ ! -d "$FRONTEND_DIR" ]; then
  error_log "Frontend-Verzeichnis '$FRONTEND_DIR' nicht gefunden."
  exit 1
fi

# Prüfen ob Docker installiert ist
if ! command -v docker &> /dev/null; then
  error_log "Docker ist nicht installiert. Bitte installieren Sie Docker."
  log "Siehe: https://docs.docker.com/get-docker/"
  exit 1
fi

if ! command -v docker compose &> /dev/null; then
  error_log "Docker Compose ist nicht installiert. Bitte installieren Sie Docker Compose."
  log "Siehe: https://docs.docker.com/compose/install/"
  exit 1
fi

# Prüfen ob wget installiert ist (wird für healthcheck verwendet)
if ! command -v wget &> /dev/null; then
  error_log "wget ist nicht installiert, wird aber für den Container-Healthcheck benötigt."
  log "Installation mit: sudo apt-get install wget"
  exit 1
fi

# Wechsle ins Frontend-Verzeichnis
log "Wechsle ins Verzeichnis: $FRONTEND_DIR"
cd "$FRONTEND_DIR" || {
  error_log "Konnte nicht ins Verzeichnis $FRONTEND_DIR wechseln."
  exit 1
}

# Prüfe ob Dockerfile existiert
if [ ! -f "Dockerfile" ]; then
  error_log "Dockerfile nicht gefunden in $FRONTEND_DIR."
  exit 1
fi

# Prüfe ob docker-compose.yml existiert
if [ ! -f "docker-compose.yml" ]; then
  error_log "docker-compose.yml nicht gefunden in $FRONTEND_DIR."
  exit 1
fi

# Prüfe ob package.json existiert
if [ ! -f "package.json" ]; then
  error_log "package.json nicht gefunden in $FRONTEND_DIR."
  exit 1
fi

# Erstelle .env-Datei
log "${YELLOW}Erstelle .env Datei...${NC}"
cat > .env << EOL
# Basic Server Configuration
RUNTIME_ENVIRONMENT=prod
DB_PATH=db/sqlite.db

# OAuth Configuration (Bitte anpassen)
OAUTH_CLIENT_ID=client_id
OAUTH_CLIENT_SECRET=client_secret

# Backend-API URL (IP-Adresse oder Hostname des Backend-Servers)
NEXT_PUBLIC_API_URL=http://localhost:5000
EOL

if [ ! -f ".env" ]; then
  error_log "Konnte .env-Datei nicht erstellen. Prüfen Sie die Berechtigungen."
  exit 1
fi
log "${GREEN}.env Datei erfolgreich erstellt${NC}"
log "${YELLOW}HINWEIS: Bitte passen Sie die Backend-URL in der .env-Datei an, falls das Backend auf einem anderen Server läuft.${NC}"

# Datenbank-Verzeichnis erstellen
log "Erstelle Datenbankverzeichnis"
if ! mkdir -p db; then
  error_log "Konnte Verzeichnis 'db' nicht erstellen. Prüfen Sie die Berechtigungen."
  exit 1
fi

# Docker-Image bauen und starten
log "${YELLOW}Baue und starte Frontend-Container...${NC}"
if ! docker compose up -d --build; then
  error_log "Docker Compose fehlgeschlagen. Siehe Fehlermeldung oben."
  exit 1
fi

# Prüfe, ob der Container läuft
log "Warte 10 Sekunden, bis der Container gestartet ist..."
sleep 10
if docker ps | grep -q "myp-frontend"; then
  log "${GREEN}Frontend-Container läuft${NC}"
else
  error_log "Frontend-Container läuft nicht. Container-Status:"
  docker ps -a | grep myp-frontend
  log "Container-Logs:"
  docker logs myp-frontend
  exit 1
fi

# Teste ob der Server erreichbar ist
log "${YELLOW}Teste ob Frontend-Server erreichbar ist...${NC}"
MAX_ATTEMPTS=12
ATTEMPT=1

while [ $ATTEMPT -le $MAX_ATTEMPTS ]; do
  log "Versuch $ATTEMPT/$MAX_ATTEMPTS..."
  if curl -s -o /dev/null -w "%{http_code}" http://localhost:3000 | grep -q "200\|304"; then
    log "${GREEN}Frontend-Server ist erreichbar!${NC}"
    break
  else
    if [ $ATTEMPT -eq $MAX_ATTEMPTS ]; then
      log "${YELLOW}Server noch nicht erreichbar. Das ist normal, wenn der Next.js-Build länger dauert.${NC}"
      log "Überprüfen Sie den Container-Status mit: docker logs myp-frontend"
      log "Der Server sollte in wenigen Minuten verfügbar sein."
    else
      log "Server noch nicht erreichbar. Warte 10 Sekunden..."
      sleep 10
    fi
  fi
  ATTEMPT=$((ATTEMPT+1))
done

# Prüfe ob die Datenbank-Migration erfolgt ist
log "${YELLOW}Prüfe Datenbank-Migration...${NC}"
if [ -f "db/sqlite.db" ]; then
  log "${GREEN}Datenbank existiert${NC}"
  
  # Prüfe Datenbankgröße
  DB_SIZE=$(du -b db/sqlite.db | cut -f1)
  if [ "$DB_SIZE" -gt 1000 ]; then
    log "${GREEN}Datenbank scheint initialisiert zu sein (Größe: $DB_SIZE Bytes)${NC}"
  else
    log "${YELLOW}Datenbank ist sehr klein (Größe: $DB_SIZE Bytes). Möglicherweise wurde die Migration nicht korrekt ausgeführt.${NC}"
    log "Sie können die Migration manuell ausführen mit: docker exec myp-frontend pnpm db:migrate"
  fi
else
  log "${YELLOW}Datenbank-Datei nicht gefunden. Migration könnte fehlgeschlagen sein.${NC}"
  log "Führen Sie die Migration manuell aus: docker exec myp-frontend pnpm db:migrate"
  log "Prüfen Sie die Container-Logs: docker logs myp-frontend"
fi

log "${GREEN}=== Installation abgeschlossen ===${NC}"
log "Das Frontend ist unter http://localhost:3000 erreichbar"
log "Anzeigen der Logs: docker logs -f myp-frontend"
log "Frontend stoppen: docker compose -f $FRONTEND_DIR/docker-compose.yml down"